// Source          ::     RoverV2_ComponentTests_withTM1638

// Description     ::     Simple program to test the various components that come with the DFRobotShop Rover V2 Autonomous. 
//                        http://www.robotshop.com/ca/en/dfrobotshop-rover-v2-autonomous.html 
//
//                        This program invokes no complicated actions as a result of the sensor readings, 
//                        its purpose is to demonstrate the data values in relation to the environment they are interacting with.
//                         
//                        Tests are contained in seperate functions, just activate the one(s) you're interested in 
//                        by modifying the CONSTANTS found in Program Control below.

//                        Currently, the following components are being tested :        

//                        Tamiya Twin-Motor Gear Box
//                        http://www.robotshop.com/ca/en/tamiya-twin-motor-gearbox.html

//                        Encoder pair for Tamiya Twin Motor Gearbox
//                        http://www.robotshop.com/ca/en/encoder-pair-tamiya-twin-motor-gearbox.html

//                        Pan & Tilt Servos as provided with the Dagu Mini Pan and Tilt Kit
//                        http://www.robotshop.com/ca/en/dagu-mini-pan-and-tilt-kit.html

//                        DFRobot Buzzer Module
//                        http://www.robotshop.com/ca/en/dfrobot-buzzer-module.html
 
//                        Seeed Studio SEN136B5B Ultrasonic Rangefinder
//                        http://www.robotshop.com/en/seeedstudio-ultrasonic-range-finder.html                     

//                        Dagu Compound Infrared Sensor
//                        http://www.robotshop.com/en/dagu-compound-infrared-sensor.html

//                        I/O Expansion Shield V7 
//                        http://www.robotshop.com/ca/en/dfrobot-io-expansion-shield-arduino-v7.html
//                        http://www.dfrobot.com/wiki/index.php/IO_Expansion_Shield_for_Arduino_V7_SKU:DFR0265

//                        TM1638 Module (many available on eBay, link is to trusted vendor)
//                        http://www.ebay.com/usr/alice1101983



// Original Program   ::     Joe M  / arduino (at) bellaliant.net / Feb 2014

// Modified By        ::     Your Name



//  This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
//  as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  Lesser General Public License for more details.
//
//  For a copy of the GNU Lesser General Public License, 
//  write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA





/**** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T ****
/**********************************************************************************************************************************************************************/
// Program Control... IMPORTANT!!!! ... set the following CONSTANTS to 1 to test that particular component.
// This program assumes you will test ONE COMPONENT AT A TIME, although you are not constrained by this.
/**********************************************************************************************************************************************************************/

#define    BUZZER                  1        // Beep, beep

#define    GEARBOX                 0        // Individual left/right gearbox test

#define    FORWARDBACK             0        // Forward and backward every 30 seconds ( duration can be changed in function )

#define    ENCODERLEFT             0        // Displays left encoder values on Serial Monitor as a frequency distribution ( how many times each value is detected )
#define    ENCODERRIGHT            0        // Displays right encoder values on Serial Monitor as a frequency distribution ( how many times each value is detected )

#define    REVOLUTIONRIGHT         0        // What your Rover thinks "one revolution" is, right encoder
#define    REVOLUTIONLEFT          0        // What your Rover thinks "one revolution" is, left encoder

#define    SERVOPAN                0        // Test of Pan  servo, modify AnglesToTest[] and numberOfAngles at top of loop to change which angles are tested 
#define    SERVOTILT               0        // Test of Tilt servo, modify AnglesToTest[] and numberOfAngles at top of loop to change which angles are tested

#define    RANGEFINDER             0        // Use ultrasonic sensor to determine distance to closest object. Assumes buzzer connected, beeps faster as distance becomes shorter ( < 10 inches ) 

#define    INFRARED                0        // Use IR sensor to determine distance to closest object. Assumes buzzer connected, beeps faster as distance becomes shorter

//#define    TM1638INSTALLED                // Only activate this line if you have a TM1638 LED/Key module installed https://code.google.com/p/tm1638-library/
                                            // If you activate this without the TM1638 , the Rover will appear not to run since setup() will wait for input
                                            // from the device.
                                            // The TM1638 is an excellent addition to the Rover as it provides a nice onboard I/O package.
                                            
                                            

/**** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T **** **** I M P O R T A N T ****
/**********************************************************************************************************************************************************************/
// Pinouts... IMPORTANT!!!! ... set the pinouts to your hardware configuration
//
// NEVER USE Digital pins 5,6,7,8 ... they are permanently connected to the onboard motor controller on all versions of the Rover. 
//
// Digital pins 0 and 1 can only be used if you ARE NOT using Serial communications ( ie Serial.begin() ).
// Also note that having components connected to D0 and D1 will interfere with uploading your Sketch.  
//
// If you plan on someday expanding your Rover to use a second Arduino board ( to add more components, sensors, etc ), do not use (analog pins) A4 (SDA) and A5 (SCL)
// http://www.instructables.com/id/I2C-between-Arduinos/?ALLSTEPS 
// 
/**********************************************************************************************************************************************************************/

/********** Digital Pins D0 thru D13 **********/

// This program is using Serial Communications so digital pins D0 and D1 unavailable.

#define PANPIN            2          // Which pin the pan servo is connected to.  For example, here it is attached to (digital pin) D2.
                                     // Assuming you are using the I/O Expansion Shield V7 ( http://www.dfrobot.com/wiki/index.php/IO_Expansion_Shield_for_Arduino_V7_SKU:DFR0265 ) ,
                                     // configure your three sensor leads so that ORANGE is Signal and BROWN is ground.  The power RED is in the middle.
#define TILTPIN           3

#define RANGEFINDERPIN    4          // Ultrasonic rangefinder D4.

#define MOTORRIGHTSPEED   5          // Nothing connected to these pins , but the onboard motor controller uses these circuits.
#define MOTORLEFTSPEED    6
#define MOTORRIGHTDIREC   7
#define MOTORLEFTDIREC    8

#define IRLEDSPIN         9          // IR Compound Eye LEDs (digital pin)   D9.

// Pins D10, D11, D12 available UNLESS TM1638 installed...
#ifdef TM1638INSTALLED    // "ifdef's" are one of those obscure C++ features many beginning programmers don't know about.
                          // It is a neat feature that allows "conditional compilation".  In the case of this program,
                          // the various code segments using the TM1638 LED/Key module will only be included in the compiled executable
                          // if you defined TM1838INSTALLED above ( ie #define TM1638INSTALLED ).
                          // For more on "conditional compilation", go to http://www.cplusplus.com/doc/tutorial/preprocessor/

  #define TM1638STBPIN     10        // TM1638 Strobe D10.
  #define TM1638DTOPIN     11        // TM1638 Data D11.
  #define TM1638CLKPIN     12        // TM1638 Clock D12.
  
#endif

#define BUZZERPIN         13         // Buzzer D13 ( also connected to on-board LED ).




/********** Analog Pins A0 thru A5 **********/

// FYI... using analog pins for digital I/O is just the same as using digital ones.

// A0 is referred to as Pin 14     A1 is referred to as Pin 15
// A2 is referred to as Pin 16     A3 is referred to as Pin 17
// A4 is referred to as Pin 18     A5 is referred to as Pin 19

// A2 thru A5 (plus digital pin D9) are used in this program for the Dagu Compound Infrared Sensor.
// To help you with the connections, I provide this "color-coded" explanation.   
// In this explanation, I am using a ribbon cable consisting of the following colors...
// Yellow , Green , Blue , Violet , Gray , White , Black

// Holding the sensor array with the back facing you and the pins facing up, the above color sequence should apply.
// Yellow being VCC and Black being GND with Green , Blue , Violet , Gray , White between.

// On the I/O Expansion Shield V7 , the following connections apply...
// A2 - Green , A3 - Blue , A4 - Violet , A5 - Gray
// A2 (Power) - Yellow , A2 (Ground) - Black
// D9 - White



#define RIGHTENCODERPIN   A0          // Right motor encoder  A0.
#define LEFTENCODERPIN    A1          // Left motor encoder   A1.

#define IRUPPIN           A2          // IR Compound Eye Up (analog pin)      A2.
#define IRLEFTPIN         A3          // IR Compound Eye Left (analog pin)    A3.
#define IRDOWNPIN         A4          // IR Compound Eye Down (analog pin)    A4.
#define IRRIGHTPIN        A5          // IR Compound Eye Right (analog pin)   A5.  



// A4 == SDA, A5 == SCL if second Arduino board attached



     
                             


/*********************************************************************************************************************************************************************/
// Required includes
/*********************************************************************************************************************************************************************/

#include <stdio.h>
#include <Servo.h> 

 
/*********************************************************************************************************************************************************************/
// Function Prototypes
/*********************************************************************************************************************************************************************/

void buzzer ( void );
void gearbox( void );
void goForwardAndBack( void );
void encoder( int );
void oneRevolution( int );
void servo( Servo& , int , int );
void rangeFinder( void );
void infraredSensor( void );
boolean timeIsUp( unsigned long& , unsigned int );



/*********************************************************************************************************************************************************************/
// Servo objects
/*********************************************************************************************************************************************************************/

Servo panServo;
Servo tiltServo;      // create servo objects to control the pan/tilt servos 
                      // a maximum of eight servo objects can be created 
 




/*********************************************************************************************************************************************************************/
//
// Class definition for ultrasonic rangefinder
//	
//	Function:         Measure the distance to obstacles in front and print the distance value to the serial terminal.
//                        The measured distance is from the range 0 to 400cm(157 inches).
//	Hardware:         Ultrasonic Range sensor
//	Arduino IDE:      Arduino-1.0
//	Author:	          LG		
//	Date: 	          Jan 17,2013
//	Version:          v1.0 modified by FrankieChu
//	                  by www.seeedstudio.com
//
//  This library is free software; you can redistribute it and/or  modify it under the terms of the GNU Lesser General Public License 
//  as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public  License along with this library; if not, 
//  write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
//
/*********************************************************************************************************************************************************************/


class Ultrasonic {
	public:
		Ultrasonic(int pin);
                void DistanceMeasure(void);
		long microsecondsToCentimeters(void);
		long microsecondsToInches(void);
	private:
		int _pin;         // pin number of Arduino that is connected with SIG pin of Ultrasonic Ranger.
                long duration;    // the Pulse time received;
};

Ultrasonic::Ultrasonic(int pin) {
	_pin = pin;
}


void Ultrasonic::DistanceMeasure(void) {
// Begin the detection and get the pulse back signal
        pinMode(_pin, OUTPUT);
	digitalWrite(_pin, LOW);
	delayMicroseconds(2);
	digitalWrite(_pin, HIGH);
	delayMicroseconds(5);
	digitalWrite(_pin,LOW);
	pinMode(_pin,INPUT);
	duration = pulseIn(_pin,HIGH);
}

long Ultrasonic::microsecondsToCentimeters(void) {
// The measured distance from the range 0 to 400 Centimeters
	return duration/29/2;	
}


long Ultrasonic::microsecondsToInches(void) {
// The measured distance from the range 0 to 157 Inches
	return duration/74/2;	
}
 
Ultrasonic ultrasonic(RANGEFINDERPIN);  

int rangeInInches , rangeInCentimeters;


/*********************************************************************************************************************************************************************/
//  End of ultrasonic rangefinder class
/*********************************************************************************************************************************************************************/






#ifdef TM1638INSTALLED    
/*********************************************************************************************************************************************************************/
//  TM1638 Board
/*********************************************************************************************************************************************************************/
//
// https://code.google.com/p/tm1638-library/
//
// Based on a TM1638 chip, the board uses a serial communications interface to enable three-wire control of the LEDs and reading of the pushbuttons.
//

#include <TM1638.h>


// define module pins... data DTO 11, clock CLK 12, strobe STB 10
TM1638 tm1638(TM1638DTOPIN, TM1638CLKPIN, TM1638STBPIN);

String  tm1638Msg; 
String  whoAmI = "LUNO   4";  // Name your Rover, mine is Lunokhod 4

int tm1638LEDs, tm1638Buttons;


/*********************************************************************************************************************************************************************/
//  End of TM1638 definitions
/*********************************************************************************************************************************************************************/
#endif


/*********************************************************************************************************************************************************************/
//  General Global Variables
/*********************************************************************************************************************************************************************/

#define BASELINESPEED 150

byte speedRight = BASELINESPEED, speedLeft = BASELINESPEED;
char outputString[128];




/*********************************************************************************************************************************************************************/
// Main program
/*********************************************************************************************************************************************************************/

 
void setup() {
 
  byte pos=3, incr=-1; 
  
  // Define pinmodes
  #ifdef BUZZERPIN           
            pinMode(BUZZERPIN,OUTPUT);           
  #endif
  #ifdef PANPIN              
            pinMode(PANPIN,OUTPUT);              
  #endif
  #ifdef TILTPIN             
            pinMode(TILTPIN,OUTPUT);             
  #endif
  #ifdef RANGEFINDERPIN      
            pinMode(RANGEFINDERPIN,INPUT);       
  #endif
  #ifdef IRLEDSPIN      
            pinMode(IRLEDSPIN,OUTPUT);       
  #endif
  #ifdef IRUPPIN      
            pinMode(IRUPPIN,INPUT);       
  #endif
  #ifdef IRDOWNPIN      
            pinMode(IRDOWNPIN,INPUT);       
  #endif
  #ifdef IRLEFTPIN      
            pinMode(IRLEFTPIN,INPUT);       
  #endif
  #ifdef IRRIGHTPIN      
            pinMode(IRRIGHTPIN,INPUT);       
  #endif

  
    
  // If servos being tested, center them
  if (SERVOPAN)   { panServo.attach(PANPIN);   panServo.write(90);   delay(500);   panServo.detach(); }
  if (SERVOTILT)  { tiltServo.attach(TILTPIN); tiltServo.write(90);  delay(500);   tiltServo.detach(); }
  
  
  // Open communication with PC (for diagnostics, if desired)
  Serial.begin(9600);


  #ifdef TM1638INSTALLED 
  

  // Press the leftmost button to start
  do {
    
     tm1638.setLEDs(1); delay(250);
       
    // By alternating display position (pos) between 3,2,1,0 then 0,1,2,3
    // you get a "Cylon Eye" scanner.
    if ( pos <= 0 || pos >= 3) incr *= -1;  // You're at one end or the other, toggle the increment (incr) positive/negative
    pos = ( pos < 0 ) ? 3 : pos - incr;    
    tm1638.clearDisplay();
    tm1638.setDisplayToString("    PUSH",0,0);
    tm1638.setDisplayToString("-",0,pos);
    
    tm1638.setLEDs(0); delay(250);
 
  } while ( tm1638.getButtons( ) != 1 );
  
  
  // Countdown to launch
  tm1638.setDisplayToString(whoAmI,0,0);
  tm1638LEDs = 256;
  
  for (int i=0; i<9; i++) {
    tm1638.setLEDs(tm1638LEDs-1); delay(500);
    tm1638LEDs /= 2;    
  } 
  
  tm1638.clearDisplay();
  
  #endif
  
} 
 
 
void loop() { 
  
  static unsigned long timeIsNow;    // Required for running tests for a certain duration, uninitialized for now.
 
  static int anglesToTest[10] = { 1 , 90 , 180 , 1 , 45 , 135 };  // Used for the Servo tests
  static int numberOfAngles = 6;
  
  static int i;
  
  // Execute tests as per CONSTANTS as defined at top of program.
  
  if (BUZZER)                buzzer ( );
  
  if (FORWARDBACK)           goForwardAndBack ( );
  
  if (GEARBOX)               gearbox ( );
  
  if (ENCODERRIGHT)          encoder ( 0 );  // right
  if (ENCODERLEFT)           encoder ( 1 );  // left
  
  if (REVOLUTIONRIGHT)       oneRevolution ( 0 ) ;  // right
  if (REVOLUTIONLEFT)        oneRevolution ( 1 ) ;  // left
  
  if (INFRARED)              infraredSensor ( );
  
  
  
  // Here an example of running three tests for a certain amount of time.
  
  // Let's say SERVOPAN , SERVOTILT ,  and RANGEFINDER are all set to 1.  The following code would
  // run *each* test for 20 seconds (20000 millis), one after the other endlessly.
  
  // If you wanted to, you could modify the code for the above tests in a similiar manner.
  
  // Also included below is simple code showing "button overrides" on the TM1638. 
  // Press (and hold) S6 (32) , S7 (64) , or S8 (128) will go to that test and stay on that test until you release the button. 
  // Buttons (and LEDs) are mapped to powers of 2 ... S1 (1), S2 (2) , S3 (4) , S4 (8) , etc.
  // So, pressing both S2 and S4 will yield the value 10.
  
  if (SERVOPAN) {  
    
    i = 0;  
    
    #ifdef TM1638INSTALLED
    tm1638.setLEDs(128);                                                        // Set LED to indicate which test is being run
    #endif
    
    timeIsNow = millis();                                                       // Store current time 
        
    while ( !timeIsUp ( timeIsNow , 20000 ) ) {                                 // Run test until threshold met
      
      #ifdef TM1638INSTALLED
      // If a button is being pressed other than the button associated with this test, stop doing this one
      if ( tm1638Buttons = tm1638.getButtons( ) )  if ( tm1638Buttons != 128 ) break;
      #endif
      
      servo ( panServo, PANPIN, anglesToTest[ i++ ] ); 
      if ( i >= numberOfAngles ) i = 0;      
    }
    
    servo ( panServo, PANPIN, 90 );  // Center it before leaving test
  }
  
  
  
  if (SERVOTILT) {  
    
    i = 0;  
    
    #ifdef TM1638INSTALLED
    tm1638.setLEDs(64);                                                         // Set LED to indicate which test is being run
    #endif
    
    timeIsNow = millis();                                                       // Store current time  
    
    while ( !timeIsUp ( timeIsNow , 20000 ) ) {                                 // Run test until threshold met
      
      #ifdef TM1638INSTALLED
      // If a button is being pressed other than the button associated with this test, stop doing this one
      if ( tm1638Buttons = tm1638.getButtons( ) )  if ( tm1638Buttons != 64 ) break;
      #endif
      
      servo ( tiltServo, TILTPIN, anglesToTest[ i++ ] ); 
      if ( i >= numberOfAngles ) i = 0;      
    }
    
    servo ( tiltServo, TILTPIN, 90 );  // Center it before leaving test
  }
  
  
  
  if (RANGEFINDER) {
    
    #ifdef TM1638INSTALLED
    tm1638.setLEDs(32);                                                        // Set LED to indicate which test is being run
    #endif

    timeIsNow = millis();                                                      // Store current time
    
    while ( !timeIsUp ( timeIsNow , 20000 ) ) {                                // Run test until threshold met
      
      #ifdef TM1638INSTALLED
      // If a button is being pressed other than the button associated with this test, stop doing this one
      if ( tm1638Buttons = tm1638.getButtons( ) )  if ( tm1638Buttons != 32 ) break;
      #endif
      
      rangeFinder ( );    
    }      
  }
 

}



/*********************************************************************************************************************************************************************/
// Functions <--------------------------------------------------------------------------------------------------------------------------------------------------------
/*********************************************************************************************************************************************************************/


void buzzer ( void ) {
    
  // Incoming parameters : Nothing.
  
  // Responsibilities    : Tests the buzzer
  
  // Returns             : Nothing.
    
  #ifdef TM1638INSTALLED
  tm1638.setDisplayToString("[][][][]",0,0); 
  #endif
  
  analogWrite( BUZZERPIN, 255 );    delay(1000);
  
  #ifdef TM1638INSTALLED
  tm1638.clearDisplay();
  #endif
  
  analogWrite( BUZZERPIN, 0 );      delay(1000);
  
}



void gearbox( void ) {
  
  // Incoming parameters : Nothing.
  
  // Responsibilities    : Tests gearbox, left & right motors, forward & reverse, stop.
  //                       Digital pins 5 to 8 are permanently connected to the onboard motor controller on all versions of the Rover. 

  
  // Returns             : Nothing.
  
  #ifdef TM1638INSTALLED
  tm1638.clearDisplay();
  tm1638.setDisplayToString("F-",0,6); 
  #endif
  
  analogWrite (MOTORRIGHTSPEED,speedRight);   // Right motor
  digitalWrite(MOTORRIGHTDIREC,LOW);          // Forward
  
  delay(3000);
  
  analogWrite (MOTORRIGHTSPEED,0);            // Right motor, stop
  
  delay(3000);
  
  #ifdef TM1638INSTALLED
  tm1638.setDisplayToString("R-",0,6); 
  #endif
  
  analogWrite (MOTORRIGHTSPEED,speedRight);   // Right motor
  digitalWrite(MOTORRIGHTDIREC,HIGH);         // Reverse
  
  delay(3000);
    
  analogWrite (MOTORRIGHTSPEED,0);            // Right motor, stop
  
  delay(3000);
  
  
  #ifdef TM1638INSTALLED
  tm1638.clearDisplay();
  tm1638.setDisplayToString("-F",0,0); 
  #endif

  analogWrite (MOTORLEFTSPEED,speedLeft);    // Left motor 
  digitalWrite(MOTORLEFTDIREC,LOW);          // Forward
    
  delay(3000);
  
  analogWrite (MOTORLEFTSPEED,0);            // Left motor, stop
  
  delay(3000);
  
  #ifdef TM1638INSTALLED
  tm1638.setDisplayToString("-R",0,0); 
  #endif
  
  analogWrite (MOTORLEFTSPEED,speedLeft);    // Left motor
  digitalWrite(MOTORLEFTDIREC,HIGH);         // Reverse
  
  delay(3000);
    
  analogWrite (MOTORLEFTSPEED,0);            // Left motor, stop
  
  delay(3000);
  
}


void goForwardAndBack( void ) {
  
  // Incoming parameters : Nothing.
  
  // Responsibilities    : Tests gearbox, simple "go forward" & "backwards"
  //                       Displays runtime on TM1638 (every 5 seconds)
  //                       Good for determining range/battery duration
  
  // Returns             : Nothing.  This function never exits.
  
  static unsigned long now = millis()/1000;
  static int direc = LOW;
  
  digitalWrite(MOTORRIGHTDIREC,direc);        // Forward
  digitalWrite(MOTORLEFTDIREC,direc);         // Forward
  analogWrite (MOTORRIGHTSPEED,speedRight);   // Right motor
  analogWrite (MOTORLEFTSPEED,speedLeft);     // Left motor 
  
  while (1) { // cheap infinite loop
  
    if ( now != millis()/1000 ) {
      now = millis()/1000;
      
      #ifdef TM1638INSTALLED
      if ( now % 5 == 0 ) tm1638.setDisplayToDecNumber(now, 0, false);
      #endif
      
      if ( now % 30 == 0 ) {
        direc = !direc;  // Change direction every 30 seconds... change duration in if above
        digitalWrite(MOTORRIGHTDIREC,direc);        
        digitalWrite(MOTORLEFTDIREC,direc);  
      }      
    }
  }
  


  
}


void encoder( int whichEncoder ) {
  
  // Incoming parameters : int 0 for right encoder , 1 for left encoder
  
  // Responsibilities    : Read the desired encoder, and view the results manually.
  
  //                       The encoder is an IR transmitter/receiver that tries to bounce a signal off the shiney white surface of the disc.
  //                       If you read the values with no disc installed, you should get high values in the 1000 range.
  //                       If you hold the disk close the IR eyes, you'll get low values in the 100 range.
  //                       So, high values denote the disc cutouts, and low values denote the solid (reflective) disc areas.
  //                       Generally, you should find that 100-400 is a solid area, 400-800 is a cutout.
  
  //                       To better understand your decoders, remove the track and 
  //                       view the results on Serial Monitor (CTRL+SHIFT+M).
  
  // Returns             : Nothing.
  
  static int value, freq=0, freqDist[11] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ,0};

  if ( whichEncoder == 0 ) {
    digitalWrite(MOTORRIGHTDIREC,LOW);
    analogWrite (MOTORRIGHTSPEED,speedRight);     // Right motor , forward     
  }
  else {
    digitalWrite(MOTORLEFTDIREC,LOW);
    analogWrite (MOTORLEFTSPEED,speedLeft);      // Left motor , forward    
  }  

  do { value = analogRead(whichEncoder); } while ( value < 0 || value > 1024 );   // Read encoder.  Failsafe, throw out invalid values.  
  
  #ifdef TM1638INSTALLED
  sprintf(outputString, "%04d    ", value);
  tm1638.setDisplayToString(outputString,0,0);
  #endif
  
  freqDist[ value/100 ]++;  // Track frequency of values ... ie 100s - 1 , 200s - 2 , 300s - 3 , etc.

  // Display that value, plus frequency distributions to date 
  sprintf(outputString , "\nEncoder [%d] Value [%d] Frequences 0-[%3d] 1-[%3d] 2-[%3d] 3-[%3d] 4-[%3d] 5-[%3d] 6-[%3d] 7-[%3d] 8-[%3d] 9-[%3d] 10-[%3d] " 
          , whichEncoder , value  
          , freqDist[0], freqDist[1], freqDist[2], freqDist[3], freqDist[4], freqDist[5], freqDist[6], freqDist[7], freqDist[8], freqDist[9], freqDist[10] );               
  Serial.println(outputString); 
  


  
  return;
  
}



void oneRevolution( int whichEncoder ) {
  
  // Incoming parameters : int 0 for right motor , 1 for left encoder
  
  // Responsibilities    : Tune this to get a consistent single revoltion.
  
  //                       It is assumed you understand your encoder by playing with the encoder() function
  
  // Returns             : Nothing.
  
  static int value, currentState=0, previousState=0, segment=0, revLeft=0, revRight=0;
  
  if ( whichEncoder == 0 ) {
    digitalWrite(MOTORRIGHTDIREC,LOW);
    analogWrite (MOTORRIGHTSPEED,speedRight);     // Right motor , forward     
  }
  else {
    digitalWrite(MOTORLEFTDIREC,LOW);
    analogWrite (MOTORLEFTSPEED,speedLeft);      // Left motor , forward    
  }            
  
  do { value = analogRead(whichEncoder); } while ( value < 0 || value > 1024 );   // Read encoder.  Failsafe, throw out invalid values.  
 
  // We're not too interested in the full value ( ie 489 ), so we're going to simplfy things by boiling them down to a simpler number
  // sequence.  So, all 100s will turn into 1, all 200s will turn into 2, etc. 
  value /= 100 ;
  
  currentState = ( value >= 5 ) ? 1 : 0;   // Here's where we try to map the 10 possible values ( 0 thru 10 ) to "solid" (1) or "cutout" areas.
                                           // On my encooder wheel there are 10 solid areas and 10 cutouts, so theoritically there are 20 state changes 
                                           // per revolution.   Unfortunately, since the components we are working with are not that precise, the reality
                                           // may not always match the theory.
  
  // "State changes" are counted as a "new" segment (either solid or cutout).
  if ( currentState != previousState ) {    
    segment++;
    previousState = currentState;
  }
  
    
  // Once you think you've hit a full revolution, "brake" the motor by throwing it a small reverse signal.  
  // If you don't brake, the motor may drift slightly thus throwing out your attempt to calculate a precise revolution.
  
  // Your challenge here is to tweak the number ( ie == ??? instead of == 20 ) to get it tuned as much as possible to halt after one complete rotation.  
  // Again, you'll rarely have perfect results, but it will certainly give you better insight into the components you are working with!

  if ( segment == 20 ) { 
    currentState = previousState = segment = 0;
    
    if ( whichEncoder == 0 ) { // Brake!
        digitalWrite(MOTORRIGHTDIREC,HIGH);
        analogWrite (MOTORRIGHTSPEED,30);
        revRight++;        
    }
    else {
        digitalWrite(MOTORLEFTDIREC,HIGH);
        analogWrite (MOTORLEFTSPEED,30);   
        revLeft++;   
    } 

    
    #ifdef TM1638INSTALLED
    // Display total rotation(s)
    sprintf(outputString, "%03d  %03d", revLeft, revRight);
    tm1638.setDisplayToString(outputString,0,0);
    #endif
    
    delay(1000);   // delay so you have a chance to see the rotational state.
  }
  
   
  return;
  
}


void servo ( Servo& whichServo , int whichPin , int whichAngle) {
    
  // Incoming parameters : A Servo object (by ref) indicating which servo to test, an int indicating the pin, and an int containing the desired angle
  
  // Responsibilities    : Positions servo at desired angle with delays
  
  // Returns             : Nothing.
  
  // Since servos suck a lot of power, you want them to be attached only as long as needed.
  // Attach, send a positional command, give it time to get there, detach, (and if desired) wait so you can see it.
  
  #ifdef TM1638INSTALLED
  tm1638.clearDisplay();
  tm1638.setDisplayToDecNumber(whichAngle, 0, false);
  #endif
  
  // Attach and swing
  whichServo.attach(whichPin);   
  whichServo.write(whichAngle);   
  
  delay(500);            // You may or may not want to fine tune this time delay
  
  whichServo.detach();   // Try running with this commented out... you may notice strange behaviour if you're running the rangeFinder test as well, 
                         //                                        as the servo is constantly drawing power
  delay(1500);            // Optional

}


void rangeFinder( void ) {
  
  // Incoming parameters : Nothing.
  
  // Responsibilities    : Uses ultrasonic rangefinder to determine distance.
  
  // Returns             : Nothing. 
  
  static byte pos=3, incr=-1;
  
  ultrasonic.DistanceMeasure();                                    // get the current signal time
  
  rangeInInches =        ultrasonic.microsecondsToInches();        // convert the time to inches
  rangeInCentimeters =   ultrasonic.microsecondsToCentimeters();   // convert the time to centimeters
	
  // Serial.println("The distance to obstacles in front is: "); 
  
  // Serial.print(rangeInInches);          Serial.println(" inch");
  
  // Serial.print(rangeInCentimeters);     Serial.println(" cm");

  delay(100);  
      
  #ifdef TM1638INSTALLED
  sprintf(outputString, "%03d  %03d", rangeInInches , rangeInCentimeters);
  tm1638.setDisplayToString(outputString,0,0); 
  #endif
  
  // Assumes buzzer connected, beeps faster as distance becomes shorter
  if ( rangeInInches < 10 ) {
    
    analogWrite( BUZZERPIN, 255 );    delay( (rangeInInches+1) * 50  );    // Comment these out if you find the buzzer annoying
    analogWrite( BUZZERPIN, 0 );      delay( (rangeInInches+1) * 50  );
    
  }
  
}



void infraredSensor( void ) {

  // Incoming parameters :  Nothing.
 
  // Responsibilities    :  Uses IR Sensor to determine distance.
  
  // Returns             :  Nothing.
  
  static byte pos=3, incr=-1;
  int irLeft, irRight, irUp, irDown;
  int distance, distanceSimplified;

    
  digitalWrite(IRLEDSPIN,HIGH);                                 // Turn on IR LEDs to read Total light (ambient + reflected IR)
  delayMicroseconds(500);                                       // Allow time for phototransistors to respond. (may not be needed)

  irLeft   = analogRead(IRLEFTPIN);                             // Total  = Ambient + LED IR reflected from object
  irRight  = analogRead(IRRIGHTPIN);
  irUp     = analogRead(IRUPPIN);  
  irDown   = analogRead(IRDOWNPIN);  

  digitalWrite(IRLEDSPIN,LOW);                                  // Turn off IR LEDs to read Ambient light (ie from indoor lighting and natural light)
  delayMicroseconds(500);                                       // Allow time for phototransistors to respond. (may not be needed)
   
  irLeft    =    irLeft   -   analogRead(IRLEFTPIN);            // Reflected IR = Total (read previously) - Ambient (read now)
  irRight   =    irRight  -   analogRead(IRRIGHTPIN);            
  irUp      =    irUp     -   analogRead(IRUPPIN);               
  irDown    =    irDown   -   analogRead(IRDOWNPIN);             

  distance = ( irLeft + irRight + irUp + irDown ) / 4;         // Distance of object is average of reflected IR
                                                               // Low values ( ie 34,35,36 denote far away , high values ie 634,635,636 denote close )
  
  // For this simple test, turn "distance" into a simple number from 7 (far) to 1 (close) 
  distanceSimplified = 7 - (distance/100);
  if ( distanceSimplified < 0 ) distanceSimplified = 0;  
    
  sprintf(outputString, "Distance (%03d - %d)  Left (%03d)  Right (%03d)  Up (%03d)  Down (%03d)", distance , distanceSimplified , irLeft, irRight , irUp , irDown);
  Serial.println(outputString);
  
  #ifdef TM1638INSTALLED
  sprintf(outputString, "%03d  %03d", distanceSimplified , distance);
  tm1638.setDisplayToString(outputString,0,0); 
  #endif
  
  // Assumes buzzer connected, beeps faster as distance becomes shorter
  // Alter the above equation, and the if statement below to modify desired distance alert
  if ( distanceSimplified < 7 ) {
    
    analogWrite( BUZZERPIN, 255 );    delay( (distanceSimplified+1) * 50  );    // Comment these out if you find the buzzer annoying
    analogWrite( BUZZERPIN, 0 );      delay( (distanceSimplified+1) * 50  );
    
  }
 
}




boolean timeIsUp( unsigned long& then , unsigned int desiredInterval) {
  
  // Incoming parameters : unsigned long (by ref) counter (in milliseconds) , unsigned int (by copy) desired cycle.
 
  // Responsibilities    : When the counter hits the cycle threshold (based on runtime), it will set the counter to current runtime, 
  //                       and inform whoever called this that "time is up".
  
  // Returns             : true if cycle threshold achieved, false otherwise
  
  unsigned long now = millis();
  boolean desiredIntervalAchieved = false;
  
  if (now - then >= desiredInterval)  {
    then = now;
    desiredIntervalAchieved = true;
  }

  return desiredIntervalAchieved;

}





